<?php
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
HighLevel Spintax Engine
Copyright 2021 Patrick Friedl, Braintrust Interactive. All Rights Reserved
pat@braintrustinteractive.com
https://localbizdominance.com
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
ini_set('display_errors', '1');
error_reporting(E_ALL ^ E_STRICT);
set_time_limit(300);
header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Credentials: true');
header('Access-Control-Allow-Methods: GET, PUT, POST, DELETE, OPTIONS');
header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
header('Cache-Control: post-check=0, pre-check=0', false);
header('Pragma: no-cache');

/* *************************************************************** */
// POST/GET HANDLER
/* *************************************************************** */
// do we have a post or get?
switch($_SERVER['REQUEST_METHOD']){
	case 'GET':
		$_req = &$_GET;
	break;
	case 'POST':
		$_req = &$_POST;
	break;
	default:
		$_req = &$_GET;
}
// if it's a post and we still have query string parameters, grab them too!
if($_SERVER['REQUEST_METHOD'] == 'POST' && !empty($_SERVER['QUERY_STRING'])){
	parse_str($_SERVER['QUERY_STRING'], $qs);
	foreach($qs as $key => $value){
		$_req[$key] = $value;
	}
}
// input object for raw json post
$inboundData = file_get_contents('php://input');
if(!empty($inboundData)){
	// array to add to $_req
	$json = json_decode($inboundData,TRUE);
	if(json_last_error() == JSON_ERROR_NONE){
		foreach($json as $key => $value){
			$_req[$key] = $value;
		}
	}
}

// for GHL, grab the post data
$input = json_decode(json_encode($_POST));
if(empty($input)){
	$input = json_decode($inboundData);
}
if(empty($input)){
	$input = null;
}

// set GHL api enpoints
$ghlAPIBase = 'https://rest.gohighlevel.com/v1/';
$ghlContactUpdate  = $ghlAPIBase . 'contacts/';
$ghlCustomFieldGet = $ghlAPIBase . 'custom-fields/';
$ghlCustomValueGet = $ghlAPIBase . 'custom-values/';

// fetch values from a request key
function getRequest($req = '',$fallback = null){
	global $_req;
	$val = (!empty($_req[$req]))? $_req[$req] : null;
	if(!empty($val)){
		$val = (!is_array($val)) ? trim($val) : $val;
	}
	if(empty($val)){
		return $fallback;
	} else {
		return $val;
	}
}

// GHL api handler
function doGHLCurl($endpoint = null, $token = null, $payload = null, $method = 'POST', $headers = array()){
	$method    = strtoupper($method);
	$headers[] = 'Authorization: Bearer ' . $token;
	$result    = null;
	if($method == 'POST' || $method == 'PUT' || $method == 'DELETE'){
		$json = json_encode($payload);
		$headers[] = 'Content-Type: application/json';
		$headers[] = 'Content-Length: ' . strlen($json);
		$ch = curl_init($endpoint);
		if($method == 'POST'){ curl_setopt($ch, CURLOPT_POST, true); }
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
	} else {
		$url = $endpoint;
		if(!empty($payload)){
			if(strpos($url, '?') !== false){
				$url .= '&' . http_build_query($payload);
			} else {
				$url .= '?' . http_build_query($payload);
			}
		}
		$ch = curl_init($url);
	}
	// curl_setopt($ch, CURLOPT_USERAGENT,$ua);
	curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 2);
	// fire away!
	$result = curl_exec($ch);
	curl_close($ch);
	return $result;
}

// spintax class.
class Spintax {
	public function process($text) {
        return preg_replace_callback(
            '/\[(((?>[^\[\]]+)|(?R))*)\]/x',
            array($this, 'replace'),
            $text
        );
    }
    public function replace($text) {
        $text = $this->process($text[1]);
        $parts = explode('|', $text);
        shuffle($parts);
        return $parts[array_rand($parts)];
    }
}
?>