<?php
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
HighLevel Spintax Engine
Copyright 2021 Patrick Friedl, Braintrust Interactive. All Rights Reserved
pat@braintrustinteractive.com
https://localbizdominance.com
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

// import the config file
require dirname(__FILE__) . '/config.php';

// set up our variables
$output  = array(
	'status' => 0,
	'msg' => null
);
$spintax = null;
$subject = null;
$content = null;
$spun_subject = null;
$spun_content = null;
$userFields = array(
	'contact.spun_subject' => array('id' => null, 'name' => null, 'val' => null),
	'contact.spun_content' => array('id' => null, 'name' => null, 'val' => null)
);

// check to make sure we have webhook data
if(!empty($input)){

	// check to make sure we have the required fields
	if(
		!empty($input->api_key) &&
		!empty($input->subject_field) &&
		!empty($input->content_field) &&
		!empty($input->location->id)
	){

		// set the variables we'll be using
		$lid         = $input->location->id;
		$cid         = $input->contact_id;
		$apiKey      = trim($input->api_key);
		$spinSubject = '{{custom_values.' . trim($input->subject_field) . '}}';
		$spinContent = '{{custom_values.' . trim($input->content_field) . '}}';

		// get custom fields list from GHL
		$customFields = json_decode(doGHLCurl($ghlCustomFieldGet, $apiKey, array(), 'get'));

		// cycle through the user fields and merge field keys with field IDs & names
		foreach($customFields->customFields as $cf){
			if(!empty($userFields[$cf->fieldKey])){
				$userFields[$cf->fieldKey]['id'] = $cf->id;
				$userFields[$cf->fieldKey]['name'] = $cf->name;
			}
		}

		// do the required custom fields exist?
		if(
			!empty($userFields['contact.spun_content']['id']) &&
			!empty($userFields['contact.spun_subject']['id'])
		){

			// get custom values list and get spintax field
			$customValues = json_decode(doGHLCurl($ghlCustomValueGet, $apiKey, array(), 'get'));

			// get subject to spin
			foreach($customValues->customValues as $cv){
				if($spinSubject == $cv->fieldKey){
					$subject = $cv->value;
					break;
				}
			}

			// get content to spin
			foreach($customValues->customValues as $cv){
				if($spinContent == $cv->fieldKey){
					$content = $cv->value;
					break;
				}
			}

			// do we have spintax?
			if(!empty($content) && !empty($subject)){

				// STEP 1: PROCESS CUSTOM FIELDS & CUSTOM VALUES
				// subject: process custom values - recurse to get nested values
				while(strpos($subject,'custom_values.') !== false){
					foreach($customValues->customValues as $cv){
						$key = $cv->fieldKey;
						$val = $cv->value;
						$subject = str_replace($key, $val, $subject);
						$key = str_replace('{{','{{ ',$key);
						$key = str_replace('}}',' }}',$key);
						$subject = str_replace($key, $val, $subject);
					}
				}

				// content: process custom values - recurse to get nested values
				while(strpos($content,'custom_values.') !== false){
					foreach($customValues->customValues as $cv){
						$key = $cv->fieldKey;
						$val = $cv->value;
						$content = str_replace($key, $val, $content);
						$key = str_replace('{{','{{ ',$key);
						$key = str_replace('}}',' }}',$key);
						$content = str_replace($key, $val, $content);
					}
				}

				// process user custom fields
				$subject = processCustomFields($subject,$input,$customFields);
				$content = processCustomFields($content,$input,$customFields);

				// STEP 2: SPIN!
				$spintax = new Spintax();
				$spun_subject = trim($spintax->process($subject));
				$spun_content = trim($spintax->process($content));
				$spintax = null;

				// empty results?
				$spun_subject = (empty($spun_subject))? null : $spun_subject;
				$spun_content = (empty($spun_content))? null : $spun_content;

				// STEP 3: UPDATE USER SPUN CONTENT FIELDS
				$payload = array(
					'customField' => array(
						$userFields['contact.spun_content']['id'] => $spun_content,
						$userFields['contact.spun_subject']['id'] => $spun_subject
					)
				);
				$r = json_decode(doGHLCurl($ghlContactUpdate . $cid, $apiKey, $payload, 'put'));

				$output['status'] = 1;
				$output['msg'] = 'Contact ' . $cid . ' updated';

			} else {

				// something went wrong, so empty the user's spun content fields.
				$payload = array(
					'customField' => array(
						$userFields['contact.spun_content']['id'] => null,
						$userFields['contact.spun_subject']['id'] => null
					)
				);
				$r = json_decode(doGHLCurl($ghlContactUpdate . $cid, $apiKey, $payload, 'put'));

				$output['msg'] = 'Fields ' . $spinSubject . ' and/or ' . $spinContent . ' not found, or fields are empty.';

			} // got spintax content?

		} else {

			$output['msg'] = 'Custom Field "Spun Subject" and/or "Spun Content" not found.';

		} // got spun subject/content custom fields?

	} else {

		$output['msg'] = 'Missing API Key, Location ID and/or Subject/Content fields.';

	} // got all the deets?

} else {

	$output['msg'] = 'No data posted';

} // got input

// process user custom fields in spin subject & content
function processCustomFields($data, $input, $customFields){
	// recreate default custom fields if they're empty
	if(empty($input->name)){ $input->name = ' '; }
	if(empty($input->first_name)){ $input->first_name = ' '; }
	if(empty($input->last_name)){ $input->last_name = ' '; }
	if(empty($input->email)){ $input->email = ' '; }
	if(empty($input->phone)){ $input->phone = ' '; }
	if(empty($input->company_name)){ $input->company_name = ' '; }
	if(empty($input->full_address)){ $input->full_address = ' '; }
	if(empty($input->address1)){ $input->address1 = ' '; }
	if(empty($input->city)){ $input->city = ' '; }
	if(empty($input->state)){ $input->state = ' '; }
	if(empty($input->postal_code)){ $input->postal_code = ' '; }
	if(empty($input->date_of_birth)){ $input->date_of_birth = ' '; }
	if(empty($input->source)){ $input->source = ' '; }
	if(empty($input->website)){ $input->website = ' '; }
	if(empty($input->date_of_last_stay)){ $input->date_of_last_stay = ' '; }
	if(empty($input->additional_comments)){ $input->additional_comments = ' '; }
	if(empty($input->villa_of_last_stay)){ $input->villa_of_last_stay = ' '; }

	// loop through default input fields and replace the tokens with values
	$vars = get_object_vars($input);
	foreach($vars as $key => $value){
	  if(gettype($value) == 'string'){
	  	$data = str_replace('{{contact.' . $key . '}}',trim($value),$data);
	  	$data = str_replace('{{ contact.' . $key . ' }}',trim($value),$data);
	  }
	}
	// loop through custom field array and replace tokens with contact's custom field values
	foreach($customFields->customFields as $cf){
		if(!empty($input->{$cf->name})){
			$data = str_replace('{{' . $cf->fieldKey . '}}',$input->{$cf->name},$data);
			$data = str_replace('{{ ' . $cf->fieldKey . ' }}',$input->{$cf->name},$data);
		}
	}
	// loop through custom field array and replace any leftover tokens with nothing
	// if the contact doesn't have a value, it's better to have nothing than
	// embarrassing tokens in your email showing it's a template!
	foreach($customFields->customFields as $cf){
		$data = str_replace('{{' . $cf->fieldKey . '}}','',$data);
		$data = str_replace('{{ ' . $cf->fieldKey . ' }}','',$data);
	}
	// return the populated data
	return $data;
}

// output the results to GHL's webhook
echo json_encode($output);
die();
?>